Components.utils.import("resource:///modules/iteratorUtils.jsm"); // for fixIterator

function GCClassInheriter()
{
}

GCClassInheriter.copyUndefProperties = function(src, dest)
{
	for (var prop in src) {
		if (typeof(dest[prop]) == "undefined") { 
  	  dest[prop] = src[prop];
		}
	}
}

GCClassInheriter.inherit = function(subClass, superClass) 
{
	GCClassInheriter.copyUndefProperties(superClass, subClass);
	GCClassInheriter.copyUndefProperties(superClass.prototype, subClass.prototype);
}

function GCUninstallObserver(extId, prefPrefix)
{
	this.uninstalled = false;
	this.observerService = Components.classes["@mozilla.org/observer-service;1"].
	  							getService(Components.interfaces.nsIObserverService);
	this.extensionId = extId;
	this.query = prefPrefix;
	this.pref = Components.classes['@mozilla.org/preferences;1'].
	  						getService(Components.interfaces.nsIPrefBranch);
}

GCUninstallObserver.prototype.observe = function(aSubject, aTopic, aData)
{
	switch (aTopic) {
	  case "app-startup":
			this.observerService.addObserver(this, "em-action-requested", false);
      break;
    case "em-action-requested":
	    if (aSubject instanceof Components.interfaces.nsIUpdateItem 
	    		&& aSubject.id == this.extensionId) {
				this.analyzeAction(aData);
      }
      break;
		case "quit-application":
	    this.observerService.removeObserver(this, "em-action-requested");
      if (this.uninstalled) {
      	this.removeWorkDir();
      	this.clearPrefs();
      }
      break;
	}
}

GCUninstallObserver.prototype.analyzeAction = function(aData) {
	switch (aData) {
	  case "item-disabled":
      break;
		case "item-uninstalled":
	    this.uninstalled = true;
      break;
    case "item-cancel-action":
      this.uninstalled = false;
      break;
	}	
}

GCUninstallObserver.prototype.clearPrefs = function(query) {
	if (!query) query = this.query;
	if (!query) return;
	
	try {
 		var prefCount = { value: 0 };
  	var prefArray = this.pref.getChildList("", prefCount);
  	for (var i = 0; i < prefCount.value; i++) {
    	var prefName = prefArray[i];
      var re = new RegExp("^"+query);
    	if (re.test(prefName)) {
    		this.pref.clearUserPref(prefName);
    	}
  	}
	} catch(e) {
  }
}

GCUninstallObserver.prototype.clearPref = function(prefName) {
	try {
		this.pref.clearUserPref(prefName);
	} catch(e) {
	}
}

GCUninstallObserver.prototype.removeWorkDir = function() {
	var file = null;
	try {
		var value = this.pref.getComplexValue("gmcont.dump_rel_path", Components.interfaces.nsIRelativeFilePref);
		file = value.file;
		if (!file || !file.isWritable() || !file.isReadable()) file = null;
	} catch(e) {
		file = null;
	}
	
	try {
		if (!file) {
			file = Components.classes["@mozilla.org/file/directory_service;1"]
								.getService(Components.interfaces.nsIProperties)
								.get("ProfD", Components.interfaces.nsILocalFile);
			file.append("GoogleContacts");
		}
	} catch(e) {
		dump("Error: Cannot get dump dir.\n");
		file = null;
	}
	
	if (file && file.exists()) {
		file.remove(true);
	}
}

var gGCDirManager = {
	pref: Components.classes['@mozilla.org/preferences;1'].
	  						getService(Components.interfaces.nsIPrefBranch),
	getWorkingDir: function() {
		var file = null;
		try {
			var value = this.pref.getComplexValue("gmcont.dump_rel_path", Components.interfaces.nsIRelativeFilePref);
			file = value.file;
			if (!file || !file.isWritable() || !file.isReadable()) file = null;
		} catch(e) {
			file = null;
		}
		
		try {
			if (!file) {
				file = Components.classes["@mozilla.org/file/directory_service;1"]
									.getService(Components.interfaces.nsIProperties)
									.get("ProfD", Components.interfaces.nsILocalFile);
				file.append("GoogleContacts");
			}
		} catch(e) {
			dump("Error: Cannot get dump dir.\n");
			file = null;
		}
		
		return file;
	}
};

var gGCLogger = {
	cache: [],
	cacheMax: 50,
	logFile: null,
	closed: false,
	pref: Components.classes['@mozilla.org/preferences;1'].
	  						getService(Components.interfaces.nsIPrefBranch),
	open: function() {
		try {
			this.logFile = this.pref.getComplexValue("gmcont.log_file", Components.interfaces.nsILocalFile);
			this.closed = false;
		} catch (e) {
			this.logFile = null;
			dump(e+"\n");
		}
		
		if (!this.logFile) {
			try {
				dump("Log file does not exist. Create it.\n")
				this.logFile = gGCDirManager.getWorkingDir();
				this.logFile.append("gc.log");
				if (this.logFile.exists()) {
					this.logFile.remove(true); //clear previously created log file
				}
				this.logFile.create(this.logFile.NORMAL_FILE_TYPE, 0666);
				this.pref.setComplexValue("gmcont.log_file", Components.interfaces.nsILocalFile, this.logFile.QueryInterface(Components.interfaces.nsILocalFile));
			} catch (e) {
				dump("Cannot create a log file\n");
				dump(e+"\n");
				this.logFile = null;
				return false;
			}
		}
	},
	
	close: function() {
		this.flush();
		this.closed = true;
		try {
			this.pref.clearUserPref("gmcont.log_file");
		} catch (e) {
			dump(e+"\n");
		}
	},
	
	dump: function(str) {
		dump(str);

		if (!nsPreferences.getBoolPref("gmcont.take_log", false)) return false;
		
		this.cache.push(str);
		if (this.cache.length >= this.cacheMax) {
			this.flush();
		}
		
		return true;
	},
	
	flush: function() {
		if (!this.logFile || this.closed) return false;
		
		var str = this.cache.join("");
		this.cache = [];
		
		var fileStream = Components.classes['@mozilla.org/network/file-output-stream;1']
											.createInstance(Components.interfaces.nsIFileOutputStream);
		fileStream.init(this.logFile, 0x02 | 0x10, 0x200, false); //PR_APPEND: 0x10, PR_SYNC: 0x40
		
		var converterStream = Components
				.classes['@mozilla.org/intl/converter-output-stream;1']
				.createInstance(Components.interfaces.nsIConverterOutputStream);
				
		try {										 
			converterStream.init(fileStream, "UTF-8", 0,
												 Components.interfaces.nsIConverterInputStream.DEFAULT_REPLACEMENT_CHARACTER);
			converterStream.writeString(str);
			converterStream.close();
			fileStream.close();
		} catch(e) {
			dump(e);
			return false;
		}
	}
};

addEventListener("load", function(){gGCLogger.open();}, false);
addEventListener("unload", function(){gGCLogger.flush();}, false);

gGCAccountManager = {
	passTable: [],
	getAllSignons: function() {
		this.renewPassTable();
		var userInfo = [];
		userInfo = this.getGmailSignons(userInfo);
		userInfo = this.getCustomSignons(userInfo);
		
		return userInfo;
	},
	
	renewPassTable: function() {
		this.clearPassTable();
		var passwordmanager = Components.classes["@mozilla.org/login-manager;1"]
                        .getService(Components.interfaces.nsILoginManager);
		var signons = passwordmanager.getAllLogins({});
		for (var i=0; i<signons.length; i++) {
			var nextPassword = signons[i];
	    var host = nextPassword.hostname;
	    var user;
	    var password;
	    // try/catch in case decryption fails (invalid signon entry)
	    try {
	    	user = nextPassword.username;
	      password = nextPassword.password;
	    } catch (e) {
	      // hide this entry
	      gGCLogger.dump("could not decrypt user/password for host " + host + "\n");
	      continue;
	    }

	    if (host == 'chrome://gmcont/'+user) {
	   		this.passTable[user] = password;
	    } else if (user && !this.passTable[user]) {
	    	this.passTable[user] = password;
	    }
	  }
	},
	
	clearPassTable: function() {
		this.passTable = [];
	},
	
	getGmailSignons: function(base) {
		var userInfo = base ? base : [];
		var am = Components.classes["@mozilla.org/messenger/account-manager;1"].
			getService(Components.interfaces.nsIMsgAccountManager);
		//var allServers=am.allServers;
		//var serverCount = allServers.Count();
		//var allServers = [x for each (x in fixIterator(am.allServers, Components.interfaces.nsIMsgIncomingServer))];
		var allServers = gGCUtils.enumToArray(am.allServers, Components.interfaces.nsIMsgIncomingServer);
		//for (var i = 0; i < serverCount; i++) {
		for (var i = 0; i < allServers.length; i++) {
			//var server = allServers.QueryElementAt(i, Components.interfaces.nsIMsgIncomingServer);
			var server = allServers[i];
			//not email
			if (server.type == "nntp" || server.type == "rss" || server.type == "none") continue;
			//not google
			var hostName = server.realHostName ? server.realHostName : server.hostName;
			if (!hostName.match(/(?:g|google)mail\.com$/)) continue;
		  var user = server.realUsername ? server.realUsername : server.username;
		  if (user.indexOf("@") < 0) {
		  	var hostParts = hostName.split(".");
		  	user = user + "@"
		  			 + hostParts[hostParts.length - 2] 
		  			 + "." + hostParts[hostParts.length - 1];
		 	}
			user = user.replace(/^recent:/,"");
	  	var password = this.passTable[user];
	  	if (!password) password = server.password; //can retrieve password if the server is already authed
			userInfo[user] = password;
		}
		
		return userInfo;
	},
	
	getCustomSignons: function(base) {
		var userInfo = base ? base : [];
		var customAccounts = nsPreferences.copyUnicharPref("gmcont.custom_accounts", null);
		if (customAccounts) {
			var accounts = customAccounts.split(",");
			for (var i=0; i<accounts.length; i++) {
				var account = accounts[i];
				userInfo[account] = this.passTable[account];
			}
		}
		return userInfo;
	}
};

var gGCUtils = {
	enumToArray: function(enumerator, iface) {
		var ret = [];
		if (enumerator) ret = [x for each (x in fixIterator(enumerator, iface))];
		return ret;
	}
};
